io.randomseed.utils.db.types

Opinionated database type conversions.

Opinionated database type conversions.
raw docstring

add-all-accessorsclj

(add-all-accessors)

Adds all opinionated readers and setters by calling add-all-readers and add-all-setters.

Adds all opinionated readers and setters by calling `add-all-readers` and `add-all-setters`.
sourceraw docstring

add-all-readersclj

(add-all-readers)

Adds all opinionated readers by calling add-reader-date, add-reader-blob, and reader adders registered with register-reader-adder!.

Adds all opinionated readers by calling `add-reader-date`, `add-reader-blob`, and
reader adders registered with `register-reader-adder!`.
sourceraw docstring

add-all-settersclj

(add-all-setters)

Adds all opinionated setters by calling core setter adders and setter adders registered with register-setter-adder!.

Adds all opinionated setters by calling core setter adders and setter adders
registered with `register-setter-adder!`.
sourceraw docstring

add-reader-blobclj

(add-reader-blob)

Extends next.jdbc.result-set/ReadableColumn protocol to support binary large object (BLOB) conversions, so java.sql.Blob data are converted to an array of bytes (bytes[]).

Extends `next.jdbc.result-set/ReadableColumn` protocol to support binary large
object (BLOB) conversions, so `java.sql.Blob` data are converted to an array of
bytes (`bytes[]`).
sourceraw docstring

add-reader-dateclj

(add-reader-date)

Extends next.jdbc.result-set/ReadableColumn protocol to support date/time conversions so java.sql.Date data are passed as-is and java.sql.Timestamp data are converted to java.sql.Timestamp.

Extends `next.jdbc.result-set/ReadableColumn` protocol to support date/time
conversions so `java.sql.Date` data are passed as-is and `java.sql.Timestamp`
data are converted to `java.sql.Timestamp`.
sourceraw docstring

add-setter-dateclj

(add-setter-date)

Extends next.jdbc.prepare/SettableParameter protocol to support date/time conversions so java.sql.Date data are saved using .setDate method and java.sql.Timestamp, java.time.Instant, java.time.ZonedTime, java.time.LocalDate, java.time.LocalDateTime and java.util.Date are saved with .setTimestamp. Also note that java.time.Instant, java.time.ZonedDateTime and java.util.Date are explicitly converted to UTC before saving.

Extends `next.jdbc.prepare/SettableParameter` protocol to support date/time
conversions so `java.sql.Date` data are saved using `.setDate` method and
`java.sql.Timestamp`, `java.time.Instant`, `java.time.ZonedTime`,
`java.time.LocalDate`, `java.time.LocalDateTime` and `java.util.Date` are saved with
`.setTimestamp`. Also note that `java.time.Instant`, `java.time.ZonedDateTime` and
`java.util.Date` are explicitly converted to UTC before saving.
sourceraw docstring

add-setter-uuidclj

(add-setter-uuid)

Extends next.jdbc.prepare/SettableParameter protocol to support UUID conversions so java.util.UUID data are converted to an array of bytes and then saved using .setBytes method.

Extends `next.jdbc.prepare/SettableParameter` protocol to support UUID
conversions so `java.util.UUID` data are converted to an array of bytes and then
saved using `.setBytes` method.
sourceraw docstring

adders-resource-pathclj

Classpath resource path used by optional DB adapter registration.

Classpath resource path used by optional DB adapter registration.
sourceraw docstring

opts-mapclj

Default next.jdbc options for qualified map results with snake/lisp case conversion.

Default next.jdbc options for qualified map results with snake/lisp case conversion.
sourceraw docstring

opts-simple-mapclj

Default next.jdbc options for unqualified map results with snake/lisp case conversion.

Default next.jdbc options for unqualified map results with snake/lisp case conversion.
sourceraw docstring

opts-simple-vecclj

Default next.jdbc options for unqualified vector results with snake/lisp case conversion.

Default next.jdbc options for unqualified vector results with snake/lisp case conversion.
sourceraw docstring

opts-slashed-mapclj

Default next.jdbc options for unqualified map results using slashed lisp-case labels.

Default next.jdbc options for unqualified map results using slashed lisp-case labels.
sourceraw docstring

opts-slashed-vecclj

Default next.jdbc options for unqualified vector results using slashed lisp-case labels.

Default next.jdbc options for unqualified vector results using slashed lisp-case labels.
sourceraw docstring

opts-vecclj

Default next.jdbc options for qualified vector results with snake/lisp case conversion.

Default next.jdbc options for qualified vector results with snake/lisp case conversion.
sourceraw docstring

register-reader-adder!clj

(register-reader-adder! adder)

Registers a no-arg reader-adder function (or var). Registered adders are executed by add-all-readers.

Registers a no-arg reader-adder function (or var). Registered adders are executed
by `add-all-readers`.
sourceraw docstring

register-setter-adder!clj

(register-setter-adder! adder)

Registers a no-arg setter-adder function (or var). Registered adders are executed by add-all-setters.

Registers a no-arg setter-adder function (or var). Registered adders are executed
by `add-all-setters`.
sourceraw docstring

to-lispclj

Memoized converter from snake_case to lisp-case (handles namespaced identifiers).

Memoized converter from snake_case to lisp-case (handles namespaced identifiers).
sourceraw docstring

to-lisp-simpleclj

Memoized converter from snake_case to lisp-case (simple, name-only).

Memoized converter from snake_case to lisp-case (simple, name-only).
sourceraw docstring

to-lisp-slashedclj

Memoized converter from snake_case to lisp-case (uses slash as separator).

Memoized converter from snake_case to lisp-case (uses slash as separator).
sourceraw docstring

to-snakeclj

Memoized converter from lisp-case to snake_case (handles namespaced identifiers).

Memoized converter from lisp-case to snake_case (handles namespaced identifiers).
sourceraw docstring

to-snake-simpleclj

Memoized converter from lisp-case to snake_case (simple, name-only).

Memoized converter from lisp-case to snake_case (simple, name-only).
sourceraw docstring

to-snake-slashedclj

Memoized converter from lisp-case to snake_case (uses slash as separator).

Memoized converter from lisp-case to snake_case (uses slash as separator).
sourceraw docstring

utc-time-zoneclj

Cached UTC TimeZone instance used for timestamp conversions.

Cached UTC `TimeZone` instance used for timestamp conversions.
sourceraw docstring

cljdoc builds & hosts documentation for Clojure/Script libraries